@@ -49,15 +49,6 @@ import companyParser
 import topBottomParser
 
 
-class _FakeParser(object):
-    """Fake parser to replace obsoleted old parsers."""
-    def __init__(self, *args, **kwds):
-        pass
-
-    def parse(self, *args, **kwds):
-        return {'data': {}}
-
-
 class _ModuleProxy:
     """A proxy to instantiate and access parsers."""
     def __init__(self, module, defaultKeys=None, oldParsers=False,
@@ -108,6 +99,18 @@ _cookie_id = 'rH1jNAkjTlNXvHolvBVBsgaPICNZbNdjVjzFwzas9JRmusdjVoqBs/Hs12NR+1WFxE
 _cookie_uu = 'su4/m8cho4c6HP+W1qgq6wchOmhnF0w+lIWvHjRUPJ6nRA9sccEafjGADJ6hQGrMd4GKqLcz2X4z5+w+M4OIKnRn7FpENH7dxDQu3bQEHyx0ZEyeRFTPHfQEX03XF+yeN1dsPpcXaqjUZAw+lGRfXRQEfz3RIX9IgVEffdBAHw2wQXyf9xdMPrQELw0QNB8dsffsqcdQemjPB0w+moLcPh0JrKrHJ9hjBzdMPpcXTH7XRwwOk='
 
 
+class _FakeURLOpener(object):
+    """Fake URLOpener object, used to return empty strings instead of
+    errors.
+    """
+    def __init__(self, url, headers):
+        self.url = url
+        self.headers = headers
+    def read(self, *args, **kwds): return ''
+    def close(self, *args, **kwds): pass
+    def info(self, *args, **kwds): return self.headers
+
+
 class IMDbURLopener(FancyURLopener):
     """Fetch web pages and handle errors."""
     def __init__(self, *args, **kwargs):
@@ -199,9 +202,12 @@ class IMDbURLopener(FancyURLopener):
             # The detection of the encoding is error prone...
             warnings.warn('Unable to detect the encoding of the retrieved '
                         'page [%s]; falling back to default latin1.' % encode)
+        ##print unicode(content, encode, 'replace').encode('utf8')
         return unicode(content, encode, 'replace')
 
     def http_error_default(self, url, fp, errcode, errmsg, headers):
+        if errcode == 404:
+            return _FakeURLOpener(url, headers)
         raise IMDbDataAccessError, {'url': 'http:%s' % url,
                                     'errcode': errcode,
                                     'errmsg': errmsg,
@@ -593,6 +599,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'tvschedule')
         return self.mProxy.airing_parser.parse(cont)
 
+    get_movie_tv_schedule = get_movie_airing
+
     def get_movie_synopsis(self, movieID):
         cont = self._retrieve(imdbURL_movie_main % movieID + 'synopsis')
         return self.mProxy.synopsis_parser.parse(cont)
