@@ -27,20 +27,368 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 # FIXME: this whole module was written in a veeery short amount of time.
 #        The code should be commented, rewritten and cleaned. :-)
 
+import re
 import warnings
+from difflib import SequenceMatcher
+from codecs import lookup
 
-from imdb.parser.common.locsql import IMDbLocalAndSqlAccessSystem, \
-                    scan_names, scan_titles, titleVariations, \
-                    nameVariations, merge_roles, scan_company_names, \
-                    soundex, filterSimilarKeywords
-from imdb.utils import normalizeTitle, normalizeName, build_title, \
+from imdb import IMDbBase
+from imdb.utils import normalizeName, normalizeTitle, build_title, \
                         build_name, analyze_name, analyze_title, \
-                        build_company_name, re_episodes, _articles
+                        canonicalTitle, canonicalName, re_titleRef, \
+                        build_company_name, re_episodes, _unicodeArticles, \
+                        analyze_company_name, re_year_index, re_nameRef
 from imdb.Person import Person
 from imdb.Movie import Movie
 from imdb.Company import Company
 from imdb._exceptions import IMDbDataAccessError, IMDbError
 
+
+# =============================
+# Things that once upon a time were in imdb.parser.common.locsql.
+
+def titleVariations(title, fromPtdf=0):
+    """Build title variations useful for searches; if fromPtdf is true,
+    the input is assumed to be in the plain text data files format."""
+    if fromPtdf: title1 = u''
+    else: title1 = title
+    title2 = title3 = u''
+    if fromPtdf or re_year_index.search(title):
+        # If it appears to have a (year[/imdbIndex]) indication,
+        # assume that a long imdb canonical name was provided.
+        titldict = analyze_title(title, canonical=1)
+        # title1: the canonical name.
+        title1 = titldict['title']
+        if titldict['kind'] != 'episode':
+            # title3: the long imdb canonical name.
+            if fromPtdf: title3 = title
+            else: title3 = build_title(titldict, canonical=1, ptdf=1)
+        else:
+            title1 = normalizeTitle(title1)
+            title3 = build_title(titldict, canonical=1, ptdf=1)
+    else:
+        # Just a title.
+        # title1: the canonical title.
+        title1 = canonicalTitle(title)
+        title3 = u''
+    # title2 is title1 without the article, or title1 unchanged.
+    if title1:
+        title2 = title1
+        t2s = title2.split(u', ')
+        if t2s[-1].lower() in _unicodeArticles:
+            title2 = u', '.join(t2s[:-1])
+    return title1, title2, title3
+
+
+re_nameIndex = re.compile(r'\(([IVXLCDM]+)\)')
+
+def nameVariations(name, fromPtdf=0):
+    """Build name variations useful for searches; if fromPtdf is true,
+    the input is assumed to be in the plain text data files format."""
+    name1 = name2 = name3 = u''
+    if fromPtdf or re_nameIndex.search(name):
+        # We've a name with an (imdbIndex)
+        namedict = analyze_name(name, canonical=1)
+        # name1 is the name in the canonical format.
+        name1 = namedict['name']
+        # name3 is the canonical name with the imdbIndex.
+        if fromPtdf:
+            if namedict.has_key('imdbIndex'):
+                name3 = name
+        else:
+            name3 = build_name(namedict, canonical=1)
+    else:
+        # name1 is the name in the canonical format.
+        name1 = canonicalName(name)
+        name3 = u''
+    # name2 is the name in the normal format, if it differs from name1.
+    name2 = normalizeName(name1)
+    if name1 == name2: name2 = u''
+    return name1, name2, name3
+
+
+try:
+    from cutils import ratcliff as _ratcliff
+    def ratcliff(s1, s2, sm):
+        """Return the Ratcliff-Obershelp value between the two strings,
+        using the C implementation."""
+        return _ratcliff(s1.encode('latin_1', 'replace'),
+                        s2.encode('latin_1', 'replace'))
+except ImportError:
+    import warnings
+    warnings.warn('Unable to import the cutils.ratcliff function.'
+                    '  Searching names and titles using the "sql"'
+                    ' data access system will be slower.')
+
+    def ratcliff(s1, s2, sm):
+        """Ratcliff-Obershelp similarity."""
+        STRING_MAXLENDIFFER = 0.7
+        s1len = len(s1)
+        s2len = len(s2)
+        if s1len < s2len:
+            threshold = float(s1len) / s2len
+        else:
+            threshold = float(s2len) / s1len
+        if threshold < STRING_MAXLENDIFFER:
+            return 0.0
+        sm.set_seq2(s2.lower())
+        return sm.ratio()
+
+
+def merge_roles(mop):
+    """Merge multiple roles."""
+    new_list = []
+    for m in mop:
+        if m in new_list:
+            keep_this = new_list[new_list.index(m)]
+            if not isinstance(keep_this.currentRole, list):
+                keep_this.currentRole = [keep_this.currentRole]
+            keep_this.currentRole.append(m.currentRole)
+        else:
+            new_list.append(m)
+    return new_list
+
+
+def scan_names(name_list, name1, name2, name3, results=0, ro_thresold=None,
+                _scan_character=False):
+    """Scan a list of names, searching for best matches against
+    the given variations."""
+    if ro_thresold is not None: RO_THRESHOLD = ro_thresold
+    else: RO_THRESHOLD = 0.6
+    sm1 = SequenceMatcher()
+    sm2 = SequenceMatcher()
+    sm3 = SequenceMatcher()
+    sm1.set_seq1(name1.lower())
+    if name2: sm2.set_seq1(name2.lower())
+    if name3: sm3.set_seq1(name3.lower())
+    resd = {}
+    for i, n_data in name_list:
+        nil = n_data['name']
+        # XXX: on Symbian, here we get a str; not sure this is the
+        #      right place to fix it.
+        if isinstance(nil, str):
+            nil = unicode(nil, 'latin1', 'ignore')
+        # Distance with the canonical name.
+        ratios = [ratcliff(name1, nil, sm1) + 0.05]
+        namesurname = u''
+        if not _scan_character:
+            nils = nil.split(', ', 1)
+            surname = nils[0]
+            if len(nils) == 2: namesurname = '%s %s' % (nils[1], surname)
+        else:
+            nils = nil.split(' ', 1)
+            surname = nils[-1]
+            namesurname = nil
+        if surname != nil:
+            # Distance with the "Surname" in the database.
+            ratios.append(ratcliff(name1, surname, sm1))
+            if not _scan_character:
+                ratios.append(ratcliff(name1, namesurname, sm1))
+            if name2:
+                ratios.append(ratcliff(name2, surname, sm2))
+                # Distance with the "Name Surname" in the database.
+                if namesurname:
+                    ratios.append(ratcliff(name2, namesurname, sm2))
+        if name3:
+            # Distance with the long imdb canonical name.
+            ratios.append(ratcliff(name3,
+                        build_name(n_data, canonical=1), sm3) + 0.1)
+        ratio = max(ratios)
+        if ratio >= RO_THRESHOLD:
+            if resd.has_key(i):
+                if ratio > resd[i][0]: resd[i] = (ratio, (i, n_data))
+            else: resd[i] = (ratio, (i, n_data))
+    res = resd.values()
+    res.sort()
+    res.reverse()
+    if results > 0: res[:] = res[:results]
+    return res
+
+
+def scan_titles(titles_list, title1, title2, title3, results=0,
+                searchingEpisode=0, onlyEpisodes=0, ro_thresold=None):
+    """Scan a list of titles, searching for best matches against
+    the given variations."""
+    if ro_thresold is not None: RO_THRESHOLD = ro_thresold
+    else: RO_THRESHOLD = 0.6
+    sm1 = SequenceMatcher()
+    sm2 = SequenceMatcher()
+    sm3 = SequenceMatcher()
+    sm1.set_seq1(title1.lower())
+    sm2.set_seq2(title2.lower())
+    if title3:
+        sm3.set_seq1(title3.lower())
+        if title3[-1] == '}': searchingEpisode = 1
+    hasArt = 0
+    if title2 != title1: hasArt = 1
+    resd = {}
+    for i, t_data in titles_list:
+        if onlyEpisodes:
+            if t_data.get('kind') != 'episode':
+                continue
+            til = t_data['title']
+            if til[-1] == ')':
+                dateIdx = til.rfind('(')
+                if dateIdx != -1:
+                    til = til[:dateIdx].rstrip()
+            if not til:
+                continue
+            ratio = ratcliff(title1, til, sm1)
+            if ratio >= RO_THRESHOLD:
+                resd[i] = (ratio, (i, t_data))
+            continue
+        if searchingEpisode:
+            if t_data.get('kind') != 'episode': continue
+        elif t_data.get('kind') == 'episode': continue
+        til = t_data['title']
+        # XXX: on Symbian, here we get a str; not sure this is the
+        #      right place to fix it.
+        if isinstance(til, str):
+            til = unicode(til, 'latin1', 'ignore')
+        # Distance with the canonical title (with or without article).
+        #   titleS      -> titleR
+        #   titleS, the -> titleR, the
+        if not searchingEpisode:
+            til = canonicalTitle(til)
+            ratios = [ratcliff(title1, til, sm1) + 0.05]
+            # til2 is til without the article, if present.
+            til2 = til
+            tils = til2.split(', ')
+            matchHasArt = 0
+            if tils[-1].lower() in _unicodeArticles:
+                til2 = ', '.join(tils[:-1])
+                matchHasArt = 1
+            if hasArt and not matchHasArt:
+                #   titleS[, the]  -> titleR
+                ratios.append(ratcliff(title2, til, sm2))
+            elif matchHasArt and not hasArt:
+                #   titleS  -> titleR[, the]
+                ratios.append(ratcliff(title1, til2, sm1))
+        else:
+            ratios = [0.0]
+        if title3:
+            # Distance with the long imdb canonical title.
+            ratios.append(ratcliff(title3,
+                        build_title(t_data, canonical=1, ptdf=1), sm3) + 0.1)
+        ratio = max(ratios)
+        if ratio >= RO_THRESHOLD:
+            if resd.has_key(i):
+                if ratio > resd[i][0]:
+                    resd[i] = (ratio, (i, t_data))
+            else: resd[i] = (ratio, (i, t_data))
+    res = resd.values()
+    res.sort()
+    res.reverse()
+    if results > 0: res[:] = res[:results]
+    return res
+
+
+def scan_company_names(name_list, name1, results=0, ro_thresold=None):
+    """Scan a list of company names, searching for best matches against
+    the given name.  Notice that this function takes a list of
+    strings, and not a list of dictionaries."""
+    if ro_thresold is not None: RO_THRESHOLD = ro_thresold
+    else: RO_THRESHOLD = 0.6
+    sm1 = SequenceMatcher()
+    sm1.set_seq1(name1.lower())
+    resd = {}
+    withoutCountry = not name1.endswith(']')
+    for i, n in name_list:
+        # XXX: on Symbian, here we get a str; not sure this is the
+        #      right place to fix it.
+        if isinstance(n, str):
+            n = unicode(n, 'latin1', 'ignore')
+        o_name = n
+        var = 0.0
+        if withoutCountry and n.endswith(']'):
+            cidx = n.rfind('[')
+            if cidx != -1:
+                n = n[:cidx].rstrip()
+                var = -0.05
+        # Distance with the company name.
+        ratio = ratcliff(name1, n, sm1) + var
+        if ratio >= RO_THRESHOLD:
+            if resd.has_key(i):
+                if ratio > resd[i][0]: resd[i] = (ratio,
+                                            (i, analyze_company_name(o_name)))
+            else:
+                resd[i] = (ratio, (i, analyze_company_name(o_name)))
+    res = resd.values()
+    res.sort()
+    res.reverse()
+    if results > 0: res[:] = res[:results]
+    return res
+
+
+try:
+    from cutils import soundex
+except ImportError:
+    warnings.warn('Unable to import the cutils.soundex function.'
+                    '  Searches of movie titles and person names will be'
+                    ' a bit slower.')
+
+    _translate = dict(B='1', C='2', D='3', F='1', G='2', J='2', K='2', L='4',
+                      M='5', N='5', P='1', Q='2', R='6', S='2', T='3', V='1',
+                      X='2', Z='2')
+    _translateget = _translate.get
+
+    def soundex(s):
+        """Return the soundex code for the given string."""
+        # Maximum length of the soundex code.
+        SOUNDEX_LEN = 5
+        if not s: return None
+        s = s.upper()
+        soundCode =  s[0]
+        for c in s[1:]:
+            cw = _translateget(c, '0')
+            if cw != '0' and soundCode[-1] != cw:
+                soundCode += cw
+        return soundCode[:SOUNDEX_LEN]
+
+
+def _sortKeywords(keyword, kwds):
+    """Sort a list of keywords, based on the searched one."""
+    sm = SequenceMatcher()
+    sm.set_seq1(keyword.lower())
+    ratios = [(ratcliff(keyword, k, sm), k) for k in kwds]
+    checkContained = False
+    if len(keyword) > 4:
+        checkContained = True
+    for idx, data in enumerate(ratios):
+        ratio, key = data
+        if key.startswith(keyword):
+            ratios[idx] = (ratio+0.5, key)
+        elif checkContained and keyword in key:
+            ratios[idx] = (ratio+0.3, key)
+    ratios.sort()
+    ratios.reverse()
+    return [r[1] for r in ratios]
+
+
+def filterSimilarKeywords(keyword, kwdsIterator):
+    """Return a sorted list of keywords similar to the one given."""
+    seenDict = {}
+    kwdSndx = soundex(keyword.encode('ascii', 'ignore'))
+    matches = []
+    matchesappend = matches.append
+    checkContained = False
+    if len(keyword) > 4:
+        checkContained = True
+    for movieID, key in kwdsIterator:
+        if key in seenDict:
+            continue
+        seenDict[key] = None
+        if checkContained and keyword in key:
+            matchesappend(key)
+            continue
+        if kwdSndx == soundex(key.encode('ascii', 'ignore')):
+            matchesappend(key)
+    return _sortKeywords(keyword, matches)
+
+
+
+# =============================
+
 _litlist = ['screenplay/teleplay', 'novel', 'adaption', 'book',
             'production process protocol', 'interviews',
             'printed media reviews', 'essays', 'other literature']
@@ -87,11 +435,15 @@ def sub_dict(d, keys):
     return dict([(k, d[k]) for k in keys if k in d])
 
 
-def get_movie_data(movieID, kindDict, fromAka=0):
+def get_movie_data(movieID, kindDict, fromAka=0, _table=None):
     """Return a dictionary containing data about the given movieID;
-    if fromAka is true, the AkaTitle table is searched."""
-    if not fromAka: Table = Title
-    else: Table = AkaTitle
+    if fromAka is true, the AkaTitle table is searched; _table is
+    reserved for the imdbpy2sql.py script."""
+    if _table is not None:
+        Table = _table
+    else:
+        if not fromAka: Table = Title
+        else: Table = AkaTitle
     m = Table.get(movieID)
     mdict = {'title': m.title, 'kind': kindDict[m.kindID],
             'year': m.productionYear, 'imdbIndex': m.imdbIndex,
@@ -168,14 +520,14 @@ def _cmpBottom(a, b):
     return _cmpTop(a, b, what='bottom 10 rank')
 
 
-class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
+class IMDbSqlAccessSystem(IMDbBase):
     """The class used to access IMDb's data through a SQL database."""
 
     accessSystem = 'sql'
 
     def __init__(self, uri, adultSearch=1, useORM=None, *arguments, **keywords):
         """Initialize the access system."""
-        IMDbLocalAndSqlAccessSystem.__init__(self, *arguments, **keywords)
+        IMDbBase.__init__(self, *arguments, **keywords)
         if useORM is None:
             useORM = ('sqlobject', 'sqlalchemy')
         if not isinstance(useORM, (tuple, list)):
@@ -275,6 +627,87 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         self._moviesubs.update(_busd)
         self.do_adult_search(adultSearch)
 
+    def _findRefs(self, o, trefs, nrefs):
+        """Find titles or names references in strings."""
+        if isinstance(o, (unicode, str)):
+            for title in re_titleRef.findall(o):
+                a_title = analyze_title(title, canonical=0)
+                rtitle = build_title(a_title, ptdf=1)
+                if trefs.has_key(rtitle): continue
+                movieID = self._getTitleID(rtitle)
+                if movieID is None:
+                    movieID = self._getTitleID(title)
+                if movieID is None:
+                    continue
+                m = Movie(title=rtitle, movieID=movieID,
+                            accessSystem=self.accessSystem)
+                trefs[rtitle] = m
+                rtitle2 = canonicalTitle(a_title.get('title', u''))
+                if rtitle2 and rtitle2 != rtitle and rtitle2 != title:
+                    trefs[rtitle2] = m
+                if title != rtitle:
+                    trefs[title] = m
+            for name in re_nameRef.findall(o):
+                a_name = analyze_name(name, canonical=1)
+                rname = build_name(a_name, canonical=1)
+                if nrefs.has_key(rname): continue
+                personID = self._getNameID(rname)
+                if personID is None:
+                    personID = self._getNameID(name)
+                if personID is None: continue
+                p = Person(name=rname, personID=personID,
+                            accessSystem=self.accessSystem)
+                nrefs[rname] = p
+                rname2 = normalizeName(a_name.get('name', u''))
+                if rname2 and rname2 != rname:
+                    nrefs[rname2] = p
+                if name != rname and name != rname2:
+                    nrefs[name] = p
+        elif isinstance(o, (list, tuple)):
+            for item in o:
+                self._findRefs(item, trefs, nrefs)
+        elif isinstance(o, dict):
+            for value in o.values():
+                self._findRefs(value, trefs, nrefs)
+        return (trefs, nrefs)
+
+    def _extractRefs(self, o):
+        """Scan for titles or names references in strings."""
+        trefs = {}
+        nrefs = {}
+        try:
+            return self._findRefs(o, trefs, nrefs)
+        except RuntimeError, e:
+            # Symbian/python 2.2 has a poor regexp implementation.
+            import warnings
+            warnings.warn('RuntimeError in '
+                    "imdb.parser.sql.IMDbSqlAccessSystem; "
+                    "if it's not a recursion limit exceeded or we're not "
+                    "running in a Symbian environment, it's a bug:\n%s" % e)
+            return (trefs, nrefs)
+
+    def _changeAKAencoding(self, akanotes, akatitle):
+        """Return akatitle in the correct charset, as specified in
+        the akanotes field; if akatitle doesn't need to be modified,
+        return None."""
+        oti = akanotes.find('(original ')
+        if oti == -1: return None
+        ote = akanotes[oti+10:].find(' title)')
+        if ote != -1:
+            cs_info = akanotes[oti+10:oti+10+ote].lower().split()
+            for e in cs_info:
+                # excludes some strings that clearly are not encoding.
+                if e in ('script', '', 'cyrillic', 'greek'): continue
+                if e.startswith('iso-') and e.find('latin') != -1:
+                    e = e[4:].replace('-', '')
+                try:
+                    lookup(e)
+                    lat1 = akatitle.encode('latin_1', 'replace')
+                    return unicode(lat1, e, 'replace')
+                except (LookupError, ValueError, TypeError):
+                    continue
+        return None
+
     def _buildNULLCondition(self, col, val):
         """Build a comparison for columns where values can be NULL."""
         if val is None:
@@ -379,7 +812,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         imdbID = movie.imdbID
         if imdbID is not None: return '%07d' % imdbID
         m_dict = get_movie_data(movie.id, self._kind)
-        titline = build_title(m_dict, canonical=1, ptdf=1)
+        titline = build_title(m_dict, ptdf=1)
         imdbID = self.title2imdbID(titline)
         # If the imdbID was retrieved from the web and was not in the
         # database, update the database (ignoring errors, because it's
@@ -455,18 +888,24 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         s_title = title_dict['title']
         if not s_title: return []
         episodeOf = title_dict.get('episode of')
-
-        if not episodeOf:
-            if not _episodes:
-                s_title_split = s_title.split(', ')
-                if len(s_title_split) > 1 and \
-                        s_title_split[-1].lower() in _articles:
-                    s_title_rebuilt = ', '.join(s_title_split[:-1])
-                    if s_title_rebuilt:
-                        s_title = s_title_rebuilt
-        else:
+        if episodeOf:
             _episodes = False
-            s_title = normalizeTitle(s_title)
+        s_title_split = s_title.split(', ')
+        if len(s_title_split) > 1 and \
+                s_title_split[-1].lower() in _unicodeArticles:
+            s_title_rebuilt = ', '.join(s_title_split[:-1])
+            if s_title_rebuilt:
+                s_title = s_title_rebuilt
+        #if not episodeOf:
+        #    if not _episodes:
+        #        s_title_split = s_title.split(', ')
+        #        if len(s_title_split) > 1 and \
+        #                s_title_split[-1].lower() in _articles:
+        #            s_title_rebuilt = ', '.join(s_title_split[:-1])
+        #            if s_title_rebuilt:
+        #                s_title = s_title_rebuilt
+        #else:
+        #    _episodes = False
         if isinstance(s_title, unicode):
             s_title = s_title.encode('ascii', 'ignore')
 
@@ -481,6 +920,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
             conditionAka = AND(AkaTitle.q.phoneticCode == soundexCode,
                             AkaTitle.q.kindID == self._kindRev['episode'])
         elif title_dict['kind'] == 'episode' and episodeOf is not None:
+            # set canonical=0 ?  Should not make much difference.
             series_title = build_title(episodeOf, canonical=1)
             # XXX: is it safe to get "results" results?
             #      Too many?  Too few?
@@ -553,9 +993,9 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                 new_res.append(r)
                 continue
             mdict = r[1]
-            aka_title = build_title(mdict, canonical=1, ptdf=1)
+            aka_title = build_title(mdict, ptdf=1)
             orig_dict = get_movie_data(r[0], self._kind)
-            orig_title = build_title(orig_dict, canonical=1, ptdf=1)
+            orig_title = build_title(orig_dict, ptdf=1)
             if aka_title == orig_title:
                 new_res.append(r)
                 continue
@@ -644,7 +1084,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         if akat:
             res['akas'] = []
             for td, note in akat:
-                nt = build_title(td, canonical=1, ptdf=1)
+                nt = build_title(td, ptdf=1)
                 if note:
                     net = self._changeAKAencoding(note, nt)
                     if net is not None: nt = net
@@ -687,7 +1127,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                             accessSystem='sql')
                 m['episode of'] = parentSeries
                 season = episode_data.get('season', 'UNKNOWN')
-                if not episodes.has_key(season): episodes[season] = {}
+                if season not in episodes: episodes[season] = {}
                 ep_number = episode_data.get('episode')
                 if ep_number is None:
                     ep_number = max((episodes[season].keys() or [0])) + 1
@@ -699,46 +1139,37 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         res = _reGroupDict(res, self._moviesubs)
         # Do some transformation to preserve consistency with other
         # data access systems.
-        #if res.has_key('plot'):
-        #    nl = []
-        #    for i in res['plot']:
-        #        if i[-1] == ')':
-        #            sauth = i.rfind('::(author: ')
-        #            if sauth != -1:
-        #                nl.append(i[sauth+11:-1] + '::' + i[:sauth])
-        #            else: nl.append(i)
-        #        else: nl.append(i)
-        #    res['plot'][:] = nl
-        # Other transformations.
         if 'quotes' in res:
             for idx, quote in enumerate(res['quotes']):
                 res['quotes'][idx] = quote.split('::')
-        if res.has_key('runtimes') and len(res['runtimes']) > 0:
+        if 'runtimes' in res and len(res['runtimes']) > 0:
             rt = res['runtimes'][0]
             episodes = re_episodes.findall(rt)
             if episodes:
                 res['runtimes'][0] = re_episodes.sub('', rt)
                 if res['runtimes'][0][-2:] == '::':
                     res['runtimes'][0] = res['runtimes'][0][:-2]
-        #if res.has_key('year'):
-        #    res['year'] = res['year']
-        if res.has_key('votes'):
+        if 'votes' in res:
             res['votes'] = int(res['votes'][0])
-        if res.has_key('rating'):
+        if 'rating' in res:
             res['rating'] = float(res['rating'][0])
-        if res.has_key('votes distribution'):
+        if 'votes distribution' in res:
             res['votes distribution'] = res['votes distribution'][0]
-        if res.has_key('mpaa'):
+        if 'mpaa' in res:
             res['mpaa'] = res['mpaa'][0]
-        if res.has_key('guest'):
-            res['guests'] = res['guest']
-            del res['guest']
         if 'top 250 rank' in res:
             try: res['top 250 rank'] = int(res['top 250 rank'])
             except: pass
         if 'bottom 10 rank' in res:
-            try: res['bottom 10 rank'] = int(res['bottom 10 rank'])
+            try: res['bottom 100 rank'] = int(res['bottom 10 rank'])
             except: pass
+            del res['bottom 10 rank']
+        for old, new in [('guest', 'guests'), ('trademarks', 'trade-mark'),
+                        ('articles', 'article'), ('pictorials', 'pictorial'),
+                        ('magazine-covers', 'magazine-cover-photo')]:
+            if old in res:
+                res[new] = res[old]
+                del res[old]
         trefs,nrefs = {}, {}
         trefs,nrefs = self._extractRefs(sub_dict(res,Movie.keys_tomodify_list))
         return {'data': res, 'titlesRefs': trefs, 'namesRefs': nrefs,
@@ -792,7 +1223,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         try:
             qr = [(q.id, {'name': q.name, 'imdbIndex': q.imdbIndex})
                     for q in Name.select(condition)]
-            
+
             q2 = [(q.personID, {'name': q.name, 'imdbIndex': q.imdbIndex})
                     for q in AkaName.select(conditionAka)]
             qr += q2
@@ -858,7 +1289,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                 duty = orig_duty = group[0][3]
                 if duty not in seenDuties: seenDuties.append(orig_duty)
                 note = mdata[2] or u''
-                if mdata[4].has_key('episode of'):
+                if 'episode of' in mdata[4]:
                     duty = 'episodes'
                     if orig_duty not in ('actor', 'actress'):
                         if note: note = ' %s' % note
@@ -883,16 +1314,11 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                 episodes[k].reverse()
             res['episodes'] = episodes
         for duty in seenDuties:
-            if res.has_key(duty):
+            if duty in res:
                 if duty in ('actor', 'actress', 'himself', 'herself',
                             'themselves'):
                     res[duty] = merge_roles(res[duty])
                 res[duty].sort()
-        # XXX: is 'guest' still needed?  I think every GA reference in
-        #      the biographies.list file was removed.
-        if res.has_key('guest'):
-            res['notable tv guest appearances'] = res['guest']
-            del res['guest']
         # Info about the person.
         pinfo = [(self._info[pi.infoTypeID], pi.info, pi.note)
                 for pi in PersonInfo.select(PersonInfo.q.personID == personID)]
@@ -918,24 +1344,14 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         # data access systems.
         for key in ('birth date', 'birth notes', 'death date', 'death notes',
                         'birth name', 'height'):
-            if res.has_key(key):
+            if key in res:
                 res[key] = res[key][0]
-        #if res.has_key('mini biography'):
-        #    nl = []
-        #    for i in res['mini biography']:
-        #        if i[-1] == ')':
-        #            sauth = i.rfind('::(author: ')
-        #            if sauth != -1:
-        #                nl.append(i[sauth+11:-1] + '::' + i[:sauth])
-        #            else: nl.append(i)
-        #        else: nl.append(i)
-        #    res['mini biography'][:] = nl
-        if res.has_key('guest'):
+        if 'guest' in res:
             res['notable tv guest appearances'] = res['guest']
             del res['guest']
         miscnames = res.get('nick names', [])
-        if res.has_key('birth name'): miscnames.append(res['birth name'])
-        if res.has_key('akas'):
+        if 'birth name' in res: miscnames.append(res['birth name'])
+        if 'akas' in res:
             for mname in miscnames:
                 if mname in res['akas']: res['akas'].remove(mname)
             if not res['akas']: del res['akas']
@@ -1073,7 +1489,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         for x in res:
             tmpd = x[1]
             country = tmpd.get('country')
-            if country is None and tmpd.has_key('country'):
+            if country is None and 'country' in tmpd:
                 del tmpd['country']
             returnl.append((x[0], tmpd))
         return returnl
